from numpy import maximum
from numpy import zeros

from gwlfe.BMPs.AgAnimal.NFEN import NFEN
from gwlfe.BMPs.AgAnimal.NFEN import NFEN_f
from gwlfe.BMPs.Stream.NSTAB import NSTAB
from gwlfe.BMPs.Stream.NSTAB import NSTAB_f
from gwlfe.BMPs.Stream.NURBBANK import NURBBANK
from gwlfe.BMPs.Stream.NURBBANK import NURBBANK_f
from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.StreamBankN import StreamBankN
from gwlfe.Output.Loading.StreamBankN import StreamBankN_f


@memoize
def StreamBankN_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                  CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                  UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                  RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                  TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                  NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                  AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                  UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42):
    result = zeros((NYrs, 12))
    streambank_n = StreamBankN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                               CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                               UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                               TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                               NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                               AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)
    nstab = NSTAB(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                  CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                  UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                  RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                  TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                  NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                  AvSlope, SedAAdjust, StreamLength, n42b, n46c, SedNitr, BankNFrac, n69c)

    nfen = NFEN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                AvSlope, SedAAdjust, StreamLength, AgLength,
                n42, SedNitr, BankNFrac, n45, n69)

    nurbbank = NURBBANK(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                        CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                        UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                        RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                        TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                        NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                        AvSlope, SedAAdjust, StreamLength, n42b, UrbBankStab, SedNitr, BankNFrac, n69c)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = streambank_n[Y][i] - (nstab[Y][i] + nfen[Y][i] + nurbbank[Y][i])
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def StreamBankN_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                    CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                    UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                    RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                    TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                    NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                    AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                    UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42):
    streambank_n = StreamBankN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                 CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                 UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                 RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                 TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                                 NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                                 AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)
    nstab = NSTAB_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                    CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                    UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                    RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                    TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                    NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                    AvSlope, SedAAdjust, StreamLength, n42b, n46c, SedNitr, BankNFrac, n69c)

    nfen = NFEN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                  CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                  UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                  RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                  TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                  NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                  AvSlope, SedAAdjust, StreamLength, AgLength,
                  n42, SedNitr, BankNFrac, n45, n69)

    nurbbank = NURBBANK_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                          CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                          UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                          RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                          TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                          NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                          AvSlope, SedAAdjust, StreamLength, n42b, UrbBankStab, SedNitr, BankNFrac, n69c)

    return maximum(streambank_n - (nstab + nfen + nurbbank), 0)



from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Input.LandUse.RurAreaTotal import RurAreaTotal
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.AvCNRur import AvCNRur
from gwlfe.MultiUse_Fxns.Runoff.AvCNUrb import AvCNUrb


@memoize
def AvCN(NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area):
    result = 0
    urbareatotal = UrbAreaTotal(NRur, NUrb, Area)
    rurareatotal = RurAreaTotal(NRur, Area)
    areatotal = AreaTotal(NRur, NUrb, Area)
    avcnurb = AvCNUrb(NRur, NUrb, CNI_0, CNP_0, Imper, Area)
    avcnrur = AvCNRur(NRur, Area, CN)
    # Calculate the average CN
    if areatotal == 0:
        result += 0
    else:
        result += ((avcnrur * rurareatotal / areatotal) + (avcnurb * urbareatotal / areatotal))
    return result


def AvCN_f(NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area):
    areatotal = AreaTotal(NRur, NUrb, Area)
    # Calculate the average CN
    if areatotal == 0:
        return 0
    else:
        urbareatotal = UrbAreaTotal(NRur, NUrb, Area)
        rurareatotal = RurAreaTotal(NRur, Area)
        avcnurb = AvCNUrb(NRur, NUrb, CNI_0, CNP_0, Imper, Area)
        avcnrur = AvCNRur(NRur, Area, CN)
        return ((avcnrur * rurareatotal / areatotal) + (avcnurb * urbareatotal / areatotal))



from .VariableUnitTest import VariableUnitTest


class TestTotAreaMeters(VariableUnitTest):

    def test_TotAreaMeters(self):
        pass  # no need to vectorize
        # z = self.z
        # np.testing.assert_array_almost_equal(
        #     TotAreaMeters.TotAreaMeters_f(),
        #     TotAreaMeters.TotAreaMeters(), decimal=7)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .test_output import TestOutput


class elm_fork_TestOutput(TestOutput):
    """
    Tests model generated output versus known
    static output.
    """
    __test__ = True

    @classmethod
    def setUpClass(self):
        super(elm_fork_TestOutput, self).setUpClass('underflow_exception_elm_fork_red.gms',
                                               'underflow_exception_elm_fork_red_output.json')



from numpy import repeat
from numpy import where
from numpy import zeros

from gwlfe.BMPs.Stream.FilterEff import FilterEff
from gwlfe.BMPs.Stream.FilterEff import FilterEff_f
from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal_f
from gwlfe.MultiUse_Fxns.Runoff.RetentionEff import RetentionEff
from gwlfe.MultiUse_Fxns.Runoff.RetentionEff import RetentionEff_f
from gwlfe.Output.Loading.SurfaceLoad import SurfaceLoad
from gwlfe.Output.Loading.SurfaceLoad import SurfaceLoad_f


@memoize
def SurfaceLoad_1(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                  Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed,
                  FilterWidth, PctStrmBuf):
    result = zeros((NYrs, 12, 31, 16, Nqual))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    nlu = NLU(NRur, NUrb)
    adjurbanqtotal = AdjUrbanQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0, CNP_0,
                                    Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    surfaceload = SurfaceLoad(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm,
                              UrbBMPRed)
    retentioneff = RetentionEff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Qretention, NRur, NUrb, Area, CNI_0,
                                AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, PctAreaInfil)
    filtereff = FilterEff(FilterWidth)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if adjurbanqtotal[Y][i][j] > 0.001:
                        for l in range(NRur, nlu):
                            for q in range(Nqual):
                                result[Y][i][j][l][q] = surfaceload[Y][i][j][l][q] * ((1 - retentioneff[Y][i][j]) * (
                                        1 - (filtereff * PctStrmBuf)))
                    else:
                        pass
                else:
                    pass
    return result


@memoize
def SurfaceLoad_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                    Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed,
                    FilterWidth, PctStrmBuf):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu - NRur, Nqual))
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adjurbanqtotal = AdjUrbanQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0, CNP_0,
                                      Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    surfaceload = SurfaceLoad_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv, Storm,
                                UrbBMPRed)
    retentioneff = RetentionEff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Qretention, NRur, NUrb, Area, CNI_0,
                                  AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, PctAreaInfil)[:, :, :, None, None]
    retentioneff = repeat(repeat(retentioneff, nlu - NRur, axis=3), Nqual, axis=4)
    filtereff = FilterEff_f(FilterWidth)
    nonzero = where((Temp > 0) & (water > 0.01) & (adjurbanqtotal > 0.001))
    result[nonzero] = surfaceload[nonzero] * (1 - retentioneff[nonzero]) * (1 - (filtereff * PctStrmBuf))
    return result



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.RuralQTotal import RuralQTotal
from gwlfe.MultiUse_Fxns.Discharge.RuralQTotal import RuralQTotal_f
# from Timer import time_function
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal_1 import UrbanQTotal_1
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal_1 import UrbanQTotal_1_f


@memoize
def QTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
           ISRR, ISRA, CN):
    result = zeros((NYrs, 12, 31))
    urban_q_total_1 = UrbanQTotal_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0, CNP_0, Imper, ISRR, ISRA)
    rural_q_total = RuralQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, NUrb, AntMoist_0, Grow_0, Area)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                # z.QTotal = 0
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    result[Y][i][j] = urban_q_total_1[Y][i][j] + rural_q_total[Y][i][j]
    return result


@memoize
def QTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
             ISRR, ISRA, CN):
    result = zeros((NYrs, 12, 31))
    urban_q_total_1 = UrbanQTotal_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0, CNP_0, Imper, ISRR, ISRA)
    rural_q_total = RuralQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, NUrb, AntMoist_0, Grow_0, Area)
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    result[where((Temp > 0) & (water > 0))] = urban_q_total_1[where((Temp > 0) & (water > 0))] + rural_q_total[
        where((Temp > 0) & (water > 0))]

    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNumImperv


class TestCNumImperv(VariableUnitTest):

    def test_CNumImperv(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNumImperv.CNumImperv_f(z.NYrs, z.NRur, z.NUrb, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.CNI_0,
                                    z.Grow_0, z.AntMoist_0),
            CNumImperv.CNumImperv(z.NYrs, z.NRur, z.NUrb, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.CNI_0, z.Grow_0,
                                  z.AntMoist_0), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Loads import NGAccManAppN


class TestNGAccManAppN(VariableUnitTest):

    def test_NGAccManAppN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGAccManAppN.NGAccManAppN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.NGPctManApp),
            NGAccManAppN.NGAccManAppN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.NGPctManApp),
            decimal=7)



from numpy import repeat
from numpy import sum
from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.NetDisLoad import NetDisLoad
from gwlfe.Output.Loading.NetDisLoad import NetDisLoad_f
from gwlfe.Output.Loading.NetSolidLoad import NetSolidLoad
from gwlfe.Output.Loading.NetSolidLoad import NetSolidLoad_f


@memoize
def Load(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
         Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
         SweepFrac, UrbSweepFrac, LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf):
    result = zeros((NYrs, 12, 3))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)

    netdisload = NetDisLoad(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                            Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                            LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    netsolidload = NetSolidLoad(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                                LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for q in range(Nqual):
                        result[Y][i][q] += netdisload[Y][i][j][q] + netsolidload[Y][i][j][q]
                        if result[Y][i][q] < 0:
                            result[Y][i][q] = 0
                else:
                    pass
    return result


@memoize
def Load_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
           Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
           LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf):
    water = repeat(Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)[:, :, :, None], Nqual, axis=3)
    temp = repeat(Temp[:, :, :, None], Nqual, axis=3)
    netdisload = NetDisLoad_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                              Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                              LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    netsolidload = NetSolidLoad_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                  Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                                  LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    result = where((temp > 0) & (water > 0.01), netdisload + netsolidload, 0)
    result = sum(result, axis=2)
    result[result < 0] = 0
    return result



from numpy import hstack
from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.LU import LU
# from Timer import time_function
from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.QrunI import QrunI, QrunI_f
from gwlfe.MultiUse_Fxns.Runoff.QrunP import QrunP, QrunP_f


# @time_function
@memoize
def UrbanQTotal(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR,
                ISRA):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urb_area_total = UrbAreaTotal(NRur, NUrb, Area)
    qrun_i = QrunI(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNI_0, AntMoist_0, Grow_0)
    qrun_p = QrunP(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)
    lu = LU(NRur, NUrb)

    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        for l in range(NRur, nlu):
                            if urb_area_total > 0:
                                result[Y][i][j] += (
                                        (qrun_i[Y][i][j][l] * (Imper[l] * (1 - ISRR[lu[l]]) * (1 - ISRA[lu[l]]))
                                         + qrun_p[Y][i][j][l] *
                                         (1 - (Imper[l] * (1 - ISRR[lu[l]]) * (1 - ISRA[lu[l]]))))
                                        * Area[l] / urb_area_total)
    return result


# @time_function
@memoize
def UrbanQTotal_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR,
                  ISRA):
    result = zeros((NYrs, 12, 31))
    urb_area_total = UrbAreaTotal(NRur, NUrb, Area)
    qrun_i = QrunI_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNI_0, AntMoist_0, Grow_0)
    qrun_p = QrunP_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)
    z = zeros((10,))
    ISRR = hstack((z, ISRR))
    ISRA = hstack((z, ISRA))
    x = (Imper * (1 - ISRR) * (1 - ISRA))
    temp = (qrun_i * x + qrun_p * (1 - x)) * Area
    if urb_area_total > 0:
        result = sum(temp, axis=3) / urb_area_total
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Rain


class TestRain(VariableUnitTest):

    def test_Rain(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Rain.Rain_f(z.Temp, z.Prec),
            Rain.Rain(z.NYrs, z.DaysMonth, z.Temp, z.Prec), decimal=7)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import os
import unittest

import numpy as np

from gwlfe import gwlfe, Parser
from gwlfe.Memoization import resetMemoization


class TestOutput(unittest.TestCase):
    """
    Tests model generated output versus known
    static output.
    """
    __test__ = False

    @classmethod
    def setUpClass(self, input_file_name, output_file_name):
        self.basepath = os.path.abspath(os.path.join(__file__, '../'))
        
        inputfile = os.path.join(self.basepath,input_file_name)
        with open(inputfile, 'r') as input_file:
            self.z = Parser.GmsReader(input_file).read()
            self.generated_output, _ = gwlfe.run(self.z)    
        
        outputfile = os.path.join(self.basepath,output_file_name)
        with open(outputfile, 'r') as output_file:
            self.static_output = json.load(output_file)    

    def test_constants(self):
        constant_keys = ["MeanFlow", "MeanFlowPerSecond", "AreaTotal"]
        error = False
        for key in constant_keys:
            try:
                self.assertIn(key, self.generated_output)
                np.testing.assert_allclose(self.generated_output[key], self.static_output[key], rtol=1e-7,
                                           verbose=True)
            except AssertionError as e:
                print("AssertionError on %s" % (key))
                print(e)
                error = True
        if (error == True):
            raise AssertionError("Not all values within margin of error")

    def test_check_monthly(self):
        self.assertEqual(len(self.generated_output["monthly"]), len(self.static_output["monthly"]))
        error = False
        for i, month in enumerate(self.generated_output["monthly"]):
            self.assertCountEqual(self.generated_output["monthly"][i], self.static_output["monthly"][i])
            for (key, val) in month.items():
                try:
                    np.testing.assert_allclose(self.generated_output["monthly"][i][key],
                                               self.static_output["monthly"][i][key],
                                               rtol=1e-7,
                                               verbose=True)
                except AssertionError as e:
                    print("AssertionError on %s (month %i)" % (key, i))
                    print(e)
                    error = True
        if (error == True):
            raise AssertionError("Not all values within margin of error")

    def test_meta(self):
        error = False
        for key in self.static_output["meta"].keys():
            try:
                self.assertIn(key, self.generated_output["meta"])
                np.testing.assert_allclose(self.generated_output["meta"][key], self.static_output["meta"][key],
                                           rtol=1e-7,
                                           verbose=True)
            except AssertionError as e:
                print("AssertionError on %s" % (key))
                print(e)
                error = True
        if (error == True):
            raise AssertionError("Not all values within margin of error")

    def test_summary_loads(self):
        self.assertEqual(len(self.generated_output["SummaryLoads"]), len(self.static_output["SummaryLoads"]))
        error = False
        for i, month in enumerate(self.generated_output["SummaryLoads"]):
            self.assertCountEqual(self.generated_output["SummaryLoads"][i], self.static_output["SummaryLoads"][i])
            for (key, val) in month.items():
                try:
                    try:
                        np.testing.assert_allclose(self.generated_output["SummaryLoads"][i][key],
                                                   self.static_output["SummaryLoads"][i][key],
                                                   rtol=1e-7,
                                                   verbose=True)
                    except TypeError:
                        self.assertEqual(self.generated_output["SummaryLoads"][i][key],
                                         self.static_output["SummaryLoads"][i][key])
                except Exception as e:
                    print("AssertionError on %s (%s)" % (key, self.generated_output["SummaryLoads"][i]["Source"]))
                    print(e)
                    error = True
        if (error == True):
            raise AssertionError("Not all values within margin of error")

    def test_loads(self):
        self.assertEqual(len(self.generated_output["Loads"]), len(self.static_output["Loads"]))
        error = False
        for i, month in enumerate(self.generated_output["Loads"]):
            self.assertCountEqual(self.generated_output["Loads"][i], self.static_output["Loads"][i])
            for (key, val) in month.items():
                try:
                    try:
                        np.testing.assert_allclose(self.generated_output["Loads"][i][key],
                                                   self.static_output["Loads"][i][key],
                                                   rtol=1e-7,
                                                   verbose=True)
                    except TypeError:
                        self.assertEqual(self.generated_output["Loads"][i][key], self.static_output["Loads"][i][key])
                except AssertionError as e:
                    print("AssertionError on %s (%s)" % (key, self.static_output["Loads"][i]["Source"]))
                    print(e)
                    error = True
        if (error == True):
            raise AssertionError("Not all values within margin of error")

    def tearDown(self):
        resetMemoization()



from .VariableUnitTest import VariableUnitTest


class TestRetentFactorN(VariableUnitTest):
    pass
    # @skip("not ready")
    # def test_RetentFactorN(self):
    #     z = self.z
    #     np.testing.assert_array_almost_equal(
    #         RetentFactorN.RetentFactorN_2(),
    #         RetentFactorN.RetentFactorN(), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Losses import GRStreamN


class TestPercolation(VariableUnitTest):
    def test_GRStreamN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRStreamN.GRStreamN_f(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                  z.AnimalDailyN),
            GRStreamN.GRStreamN(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                z.AnimalDailyN)[None, :], decimal=7)

    def test_AvGRStreamN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRStreamN.AvGRStreamN_f(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                    z.AnimalDailyN),
            GRStreamN.AvGRStreamN(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                  z.AnimalDailyN),
            decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import RetentionEff


class TestRetentionEff(VariableUnitTest):

    def test_RetentionEff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            RetentionEff.RetentionEff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Qretention, z.NRur, z.NUrb,
                                        z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                        z.PctAreaInfil),
            RetentionEff.RetentionEff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Qretention, z.NRur, z.NUrb,
                                      z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                      z.PctAreaInfil), decimal=7)



import unittest
from unittest import skip

import numpy as np

from gwlfe import Parser
from gwlfe.Input.WaterBudget import AntMoist
from .VariableUnitTest import VariableUnitTest

class TestAntMoist(VariableUnitTest):
    @skip("not ready")
    def test_AntMoist(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AntMoist.AntMoist_2(),
            AntMoist.AntMoist(), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.Animals import AEU


class TestAEU(VariableUnitTest):

    def test_AEU(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AEU.AEU_f(z.NumAnimals, z.AvgAnimalWt, z.Area),
            AEU.AEU(z.NumAnimals, z.AvgAnimalWt, z.NRur, z.NUrb, z.Area), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import GRInitBarnN


class TestGRInitBarnN(VariableUnitTest):

    def test_GRInitBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRInitBarnN.GRInitBarnN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.GRPctManApp,
                                    z.PctGrazing)[None, :],
            GRInitBarnN.GRInitBarnN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.GRPctManApp,
                                      z.PctGrazing), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import UrbQRunoff


class TestUrbQRunoff(VariableUnitTest):

    def test_UrbQRunoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            UrbQRunoff.UrbQRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                                    z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA),
            np.swapaxes(
                UrbQRunoff.UrbQRunoff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0,
                                      z.CNP_0,
                                      z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA), 1, 2)[:, :, z.NRur:]
            , decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import AvWithdrawal


class TestAvWithdrawal(VariableUnitTest):

    def test_AvWithdrawal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvWithdrawal.AvWithdrawal_f(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal),
            AvWithdrawal.AvWithdrawal(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import QTotal


class TestQTotal(VariableUnitTest):

    def test_QTotal_ground_truth(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/QTotal.npy"),
            QTotal.QTotal(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                          z.AntMoist_0,
                          z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN), decimal=7)

    # @skip("not ready")
    def test_QTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            QTotal.QTotal(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                          z.AntMoist_0,
                          z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN),
            QTotal.QTotal_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                            z.AntMoist_0,
                            z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN), decimal=7)



from numpy import average
from numpy import zeros

from .GRLostBarnN import GRLostBarnN
from .GRLostBarnN import GRLostBarnN_f
from gwlfe.Memoization import memoize


def AvGRLostBarnN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                  Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN):
    result = zeros((12,))
    gr_lost_barn_n = GRLostBarnN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                                 GRBarnNRate, Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += gr_lost_barn_n[Y][i] / NYrs
    return result


@memoize
def AvGRLostBarnN_f(NYrs, Prec, DaysMonth, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                    PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN):
    return average(
        GRLostBarnN_f(NYrs, Prec, DaysMonth, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                      PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN), axis=0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import logging
import sys
import time
import argparse
from gwlfe import gwlfe, Parser


def main():
    log = logging.getLogger('gwlfe')
    log.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    log.addHandler(ch)

    parser = argparse.ArgumentParser(description='Run the GWLF-E model for a specified gms file.')
    parser.add_argument('input', type=str, nargs="+", help='Input GMS file')
    parser.add_argument('--output', type=str, nargs="?", help='Optional output file to write result of model run')
    parser.add_argument('--json', type=str, nargs="?", help='Optional output file to write JSON result of model run')

    args = parser.parse_args()

    gms_filenames = args.input

    for gms_filename in gms_filenames:
        fp = open(gms_filename, 'r')
        z = Parser.GmsReader(fp).read()

        result, z = gwlfe.run(z)
        if (args.output != None):  # gms out filename is sepcified so write
            gmsout_filename = args.output
            log.debug("Writing GMS output file (%s)" % (gmsout_filename))
            with open(gmsout_filename, "w") as file:
                test = Parser.GmsWriter(file)
                test.write(z)

        if (args.json != None):
            with open(args.json, "w") as file:
                json.dump(result, file, indent=4, sort_keys=True)


if __name__ == '__main__':
    main()



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import SedAFactor


class TestSedAFactor(VariableUnitTest):
    def test_SedAFactor(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SedAFactor.SedAFactor_f(z.NumAnimals, z.AvgAnimalWt, z.NRur, z.NUrb, z.CNI_0, z.CNP_0, z.CN, z.Imper,
                                    z.Area, z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust),
            SedAFactor.SedAFactor(z.NumAnimals, z.AvgAnimalWt, z.NRur, z.NUrb, z.CNI_0, z.CNP_0, z.CN, z.Imper, z.Area,
                                  z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust), decimal=7)
        pass



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNumPervReten


class TestCNumPervReten(VariableUnitTest):

    def test_CNumPervReten(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNumPervReten.CNumPervReten_f(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur,
                                          z.NUrb, z.CNP_0, z.Grow_0),
            CNumPervReten.CNumPervReten(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur, z.NUrb,
                                        z.CNP_0, z.Grow_0), decimal=7)



from numpy import cumsum
from numpy import where
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.BSed import BSed
from gwlfe.MultiUse_Fxns.Erosion.BSed import BSed_f
from gwlfe.MultiUse_Fxns.Erosion.Erosion import Erosion
from gwlfe.MultiUse_Fxns.Erosion.Erosion import Erosion_f
from gwlfe.MultiUse_Fxns.Erosion.SedDelivRatio import SedDelivRatio
from gwlfe.MultiUse_Fxns.Erosion.SedTrans import SedTrans
from gwlfe.MultiUse_Fxns.Erosion.SedTrans import SedTrans_f


def SedYield(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area, NUrb, CNI_0, AntMoist_0, Grow_0,
             ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, CNP_0, Imper, SedDelivRatio_0):
    result = zeros((NYrs, 12))
    erosion = Erosion(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    bsed = BSed(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    seddelivratio = SedDelivRatio(SedDelivRatio_0)
    sedtrans = SedTrans(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                        Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    for Y in range(NYrs):
        for i in range(12):
            for m in range(i + 1):
                if bsed[Y][m] > 0:
                    result[Y][i] = result[Y][i] + erosion[Y][m] / bsed[Y][m]
            result[Y][i] = seddelivratio * sedtrans[Y][i] * result[Y][i]
    return result


@memoize
def SedYield_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area, NUrb, CNI_0, AntMoist_0,
               Grow_0,
               ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, CNP_0, Imper, SedDelivRatio_0):
    erosion = Erosion_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    bsed = BSed_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    seddelivratio = SedDelivRatio(SedDelivRatio_0)
    sedtrans = SedTrans_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                          Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    nonzero = where(bsed > 0)
    temp = zeros((NYrs, 12))
    temp[nonzero] = erosion[nonzero] / bsed[nonzero]
    return seddelivratio * sedtrans * cumsum(temp, axis=1)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import Retention


class TestRetention(VariableUnitTest):

    def test_Retention(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Retention.Retention_f(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                                  z.Grow_0),
            Retention.Retention(z.NYrs, z.DaysMonth, z.Temp, z.Prec, z.InitSnow_0, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                                z.Grow_0), decimal=7)



from numpy import sum
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.Ag.TileDrainRO import TileDrainRO
from gwlfe.Input.LandUse.Ag.TileDrainRO import TileDrainRO_f
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.AdjQTotal import AdjQTotal
from gwlfe.MultiUse_Fxns.Discharge.AdjQTotal import AdjQTotal_f
from gwlfe.MultiUse_Fxns.Discharge.QTotal import QTotal
from gwlfe.MultiUse_Fxns.Discharge.QTotal import QTotal_f


def Runoff(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
           ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity):
    result = zeros((NYrs, 12))
    adj_q_total = AdjQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                            Imper,
                            ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    q_total = QTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, CN)
    tile_drain_ro = TileDrainRO(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse,
                                Area,
                                TileDrainDensity)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if adj_q_total[Y][i][j] > 0:
                        result[Y][i] += adj_q_total[Y][i][j]
                    else:
                        result[Y][i] += q_total[Y][i][j]
                    # ADJUST THE SURFACE RUNOFF
            result[Y][i] = result[Y][i] - tile_drain_ro[Y][i]
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def Runoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
             ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity):
    adj_q_total = AdjQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper,
                              ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    q_total = QTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                       Imper,
                       ISRR, ISRA, CN)
    tile_drain_ro = TileDrainRO_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse,
                                  Area,
                                  TileDrainDensity)
    result = where(adj_q_total > 0, adj_q_total, q_total)
    result = sum(result, axis=2) - tile_drain_ro
    result[result < 0] = 0
    return result



from numpy import resize
from numpy import where
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.Erosion import Erosion
from gwlfe.MultiUse_Fxns.Erosion.Erosion import Erosion_f
from gwlfe.MultiUse_Fxns.Erosion.SedDelivRatio import SedDelivRatio
from gwlfe.Output.Loading.SedYield_1 import SedYield_1
from gwlfe.Output.Loading.SedYield_1 import SedYield_1_f


@memoize
def Erosion_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
              Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
              StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
              AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
              SedDelivRatio_0, Acoef, KF, LS, C, P):
    result = zeros((NYrs, 12))
    seddelivratio = SedDelivRatio(SedDelivRatio_0)
    erosion = Erosion(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    sedyield = SedYield_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                          Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                          RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                          StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                          AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85,
                          UrbBankStab, Acoef, KF, LS, C, P, SedDelivRatio_0)
    for Y in range(NYrs):
        for i in range(12):
            if seddelivratio > 0 and erosion[Y][i] < sedyield[Y][i]:
                result[Y][i] = sedyield[Y][i] / seddelivratio
            else:
                result[Y][i] = erosion[Y][i]
    return result


@memoize
def Erosion_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                SedDelivRatio_0, Acoef, KF, LS, C, P):
    seddelivratio = resize(SedDelivRatio(SedDelivRatio_0), (NYrs, 12))
    erosion = Erosion_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    sedyield = SedYield_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                            Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                            SatStor_0,
                            RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                            PointFlow,
                            StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                            AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85,
                            UrbBankStab, Acoef, KF, LS, C, P, SedDelivRatio_0)
    return where((seddelivratio > 0) & (erosion < sedyield), sedyield / seddelivratio, erosion)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.AvAnimalNSum import AvAnimalN


class TestAvAnimalN(VariableUnitTest):

    def test_AvAnimalN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvAnimalN.AvAnimalN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                  z.NGAppNRate, z.Prec, z.DaysMonth,
                                  z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                                  z.AWMSNgPct, z.NgAWMSCoeffN,
                                  z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct,
                                  z.GrAWMSCoeffN, z.PctStreams, z.GrazingNRate),
            AvAnimalN.AvAnimalN(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                z.NGAppNRate, z.Prec, z.DaysMonth,
                                z.NGPctSoilIncRate, z.GRPctManApp, z.GRAppNRate, z.GRPctSoilIncRate, z.NGBarnNRate,
                                z.AWMSNgPct, z.NgAWMSCoeffN,
                                z.RunContPct, z.RunConCoeffN, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct, z.GrAWMSCoeffN,
                                z.PctStreams, z.GrazingNRate), decimal=7)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Runs the GWLF-E MapShed model.

Imported from GWLF-E.frm
"""

import logging

from numpy import zeros
from numpy import seterr

from . import ReadGwlfDataFile
from . import PrelimCalculations
from . import AFOS_old
from . import CalcLoads
from . import StreamBank
from . import AnnualMeans
from . import WriteOutputFiles
from .Input.WaterBudget.InitSnow import InitSnow_f
from .Input.WaterBudget.GrowFactor import GrowFactor_f
from .Input.LandUse.TotAreaMeters import TotAreaMeters
from .Input.LandUse.Ag.AvTileDrain import AvTileDrain_f
from .Input.WaterBudget.AvWithdrawal import AvWithdrawal_f
from .Input.WaterBudget.AvGroundWater import AvGroundWater_f
from .MultiUse_Fxns.Runoff.AvRunoff import AvRunoff_f
from .Memoization import resetMemoization

log = logging.getLogger(__name__)


def run(z):
    resetMemoization()

    log.debug('Running model...')

    # Raise exception instead of printing a warning for floating point
    # overflow, underflow, and division by 0 errors.
    seterr(all='raise')

    # MODEL CALCULATIONS FOR EACH YEAR OF ANALYSIS - WATER BALANCE,
    # NUTRIENTS AND SEDIMENT LOADS
    ReadGwlfDataFile.ReadAllData(z)

    # --------- run the remaining parts of the model ---------------------

    # CALCLULATE PRELIMINARY INITIALIZATIONS AND VALUES FOR
    # WATER BALANCE AND NUTRIENTS
    PrelimCalculations.InitialCalculations(z)

    for Y in range(z.NYrs):
        # Initialize monthly septic system variables
        z.MonthPondNitr = zeros(12)
        z.MonthPondPhos = zeros(12)
        z.MonthNormNitr = zeros(12)
        z.MonthShortNitr = zeros(12)
        z.MonthShortPhos = zeros(12)
        z.MonthDischargeNitr = zeros(12)
        z.MonthDischargePhos = zeros(12)

        # FOR EACH MONTH...
        for i in range(12):
            # LOOP THROUGH NUMBER OF LANDUSES IN THE BASIN TO GET QRUNOFF
            for l in range(z.NLU):
                z.QRunoff[l, i] = 0
                z.AgQRunoff[l, i] = 0

            # DAILY CALCULATIONS
            for j in range(z.DaysMonth[Y][i]):
                # ***** END WEATHER DATA ANALYSIS *****

                # ***** WATERSHED WATER BALANCE *****

                z.PondNitrLoad = (z.NumPondSys[i] *
                                  (z.NitrSepticLoad - z.NitrPlantUptake * GrowFactor_f(z.Grow_0)[i]))
                z.PondPhosLoad = (z.NumPondSys[i] *
                                  (z.PhosSepticLoad - z.PhosPlantUptake * GrowFactor_f(z.Grow_0)[i]))

                # UPDATE MASS BALANCE ON PONDED EFFLUENT
                if (z.Temp[Y][i][j] <= 0 or InitSnow_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec)[Y][i][j] > 0):

                    # ALL INPUTS GO TO FROZEN STORAGE
                    z.FrozenPondNitr = z.FrozenPondNitr + z.PondNitrLoad
                    z.FrozenPondPhos = z.FrozenPondPhos + z.PondPhosLoad

                    # NO NUTIENT OVERFLOW
                    z.NitrPondOverflow = 0
                    z.PhosPondOverflow = 0
                else:
                    z.NitrPondOverflow = z.FrozenPondNitr + z.PondNitrLoad
                    z.PhosPondOverflow = z.FrozenPondPhos + z.PondPhosLoad
                    z.FrozenPondNitr = 0
                    z.FrozenPondPhos = 0

                # Obtain the monthly Pond nutrients
                z.MonthPondNitr[i] = z.MonthPondNitr[i] + z.NitrPondOverflow
                z.MonthPondPhos[i] = z.MonthPondPhos[i] + z.PhosPondOverflow

                # Obtain the monthly Normal Nitrogen
                z.MonthNormNitr[i] = (z.MonthNormNitr[i] + z.NitrSepticLoad -
                                      z.NitrPlantUptake * GrowFactor_f(z.Grow_0)[i])

                # 0.56 IS ATTENUATION FACTOR FOR SOIL LOSS
                # 0.66 IS ATTENUATION FACTOR FOR SUBSURFACE FLOW LOSS
                z.MonthShortNitr[i] = (z.MonthShortNitr[i] + z.NitrSepticLoad -
                                       z.NitrPlantUptake * GrowFactor_f(z.Grow_0)[i])
                z.MonthShortPhos[i] = (z.MonthShortPhos[i] + z.PhosSepticLoad -
                                       z.PhosPlantUptake * GrowFactor_f(z.Grow_0)[i])
                z.MonthDischargeNitr[i] = z.MonthDischargeNitr[i] + z.NitrSepticLoad
                z.MonthDischargePhos[i] = z.MonthDischargePhos[i] + z.PhosSepticLoad

        # CALCULATE ANIMAL FEEDING OPERATIONS OUTPUT
        AFOS_old.AnimalOperations(z, Y)

        # CALCULATE NUTRIENT AND SEDIMENT LOADS
        CalcLoads.CalculateLoads(z, Y)

        # CALCULATE STREAM BANK EROSION
        StreamBank.CalculateStreamBankEros(z, Y)

        # CALCULATE FINAL ANNUAL MEAN LOADS
        AnnualMeans.CalculateAnnualMeanLoads(z, Y)

    # CALCULATE FINAL MONTHLY AND ANNUAL WATER BALANCE FOR
    # AVERAGE STREAM FLOW

    for i in range(12):
        z.AvStreamFlow[i] = (
                AvRunoff_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                           z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                           z.n25b, z.CN, z.Landuse, z.TileDrainDensity)[i] +
                AvGroundWater_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                                z.PcntET, z.DayHrs, z.MaxWaterCap,
                                z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Landuse, z.TileDrainDensity)[i] +
                z.AvPtSrcFlow[i] +
                AvTileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                              z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                              z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                              z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                              z.Landuse, z.TileDrainDensity)[i] -
                AvWithdrawal_f(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal)[i])

        z.AvCMStream[i] = (z.AvStreamFlow[i] / 100) * TotAreaMeters(z.NRur, z.NUrb, z.Area)
        if z.AvCMStream[i] > 0:
            z.AvOrgConc[i] = (z.AvTotalOrgs[i] / (z.AvCMStream[i] * 1000)) / 10
        else:
            z.AvOrgConc[i] = 0
    z.AvOrgConc[0] = 0

    z.AvStreamFlowSum = (z.AvRunoffSum + z.AvGroundWaterSum +
                         z.AvPtSrcFlowSum + z.AvTileDrainSum -
                         z.AvWithdrawalSum)

    log.debug("Model run complete for " + str(z.NYrs) + " years of data.")

    output = WriteOutputFiles.WriteOutput(z)
    # WriteOutputFiles.WriteOutputSumFiles()
    return output, z



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns import LossFactAdj


class TestLossFactAdj(VariableUnitTest):
    def test_LossFactAdj(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            LossFactAdj.LossFactAdj(z.NYrs, z.Prec, z.DaysMonth),
            LossFactAdj.LossFactAdj_f(z.Prec, z.DaysMonth), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import InitGrN


class TestInitGrN(VariableUnitTest):
    def test_InitGrN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            InitGrN.InitGrN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            InitGrN.InitGrN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN), decimal=7)



from numpy import zeros

from gwlfe.Output.Loading.StreamBankEros import StreamBankEros
from gwlfe.Output.Loading.StreamBankEros import StreamBankEros_f


def SURBBANK(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
             ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef
             , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal
             , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope, SedAAdjust, StreamLength
             , UrbBankStab, n42b, n85d):
    result = zeros((NYrs, 12))
    streambankeros = StreamBankEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0,
                                    CNP_0, Imper,
                                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                    RecessionCoef, SeepCoef
                                    , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                                    StreamWithdrawal, GroundWithdrawal
                                    , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                                    SedAAdjust, StreamLength)
    for Y in range(NYrs):
        for i in range(12):
            if n42b > 0:
                result[Y][i] = (UrbBankStab / n42b) * streambankeros[Y][i] * n85d
    return result


def SURBBANK_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
               ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
               Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
               GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope, SedAAdjust,
               StreamLength, UrbBankStab, n42b, n85d):
    streambankeros = StreamBankEros_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0,
                                      CNP_0, Imper,
                                      ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                      RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                      TileDrainDensity,
                                      PointFlow, StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt,
                                      StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope, SedAAdjust, StreamLength)
    if n42b > 0:
        return (UrbBankStab / n42b) * streambankeros * n85d
    else:
        return zeros((NYrs, 12))



import copy

from numpy import zeros

# from Timer import time_function
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize

try:
    from .AMC5_yesterday_inner_compiled import AMC5_yesterday_inner
except ImportError:
    print("Unable to import compiled AMC5_yesterday_inner, using slower version")
    from gwlfe.Input.WaterBudget.AMC5_yesterday_inner import AMC5_yesterday_inner


# AMC5_yesterday returns the same value as yesterday(AMC5) and faster than any other version
@memoize
def AMC5(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0):
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    AMC5 = 0
    AntMoist = copy.deepcopy(AntMoist_0)
    for k in range(5):
        AMC5 += AntMoist[k]
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                AMC5 = AMC5 - AntMoist[4] + water[Y][i][j]
                AntMoist[4] = AntMoist[3]
                AntMoist[3] = AntMoist[2]
                AntMoist[2] = AntMoist[1]
                AntMoist[1] = AntMoist[0]
                AntMoist[0] = water[Y][i][j]

                result[Y][i][j] = AMC5  # TODO: why did this fix the mismatch of amc5?
    return result

@memoize
def AMC5_yesterday(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0):
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    return AMC5_yesterday_inner(NYrs, DaysMonth, AntMoist_0, water)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import AvRunoff


class TestAvRunoff(VariableUnitTest):

    def test_AvRunoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvRunoff.AvRunoff_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                                z.n25b, z.CN, z.Landuse, z.TileDrainDensity),
            AvRunoff.AvRunoff(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                              z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                              z.n25b, z.CN, z.Landuse, z.TileDrainDensity), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import WashImperv


class TestWashImperv(VariableUnitTest):

    def test_WashImperv(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            WashImperv.WashImperv_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.CNI_0, z.AntMoist_0, z.Grow_0,
                                    z.NRur, z.NUrb),
            WashImperv.WashImperv(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.CNI_0, z.AntMoist_0, z.Grow_0,
                                  z.NRur, z.NUrb), decimal=7)



from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Memoization import memoize


@memoize
def CNP(NRur, NUrb, CNP_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((3, nlu))
    for l in range(NRur, nlu):
        result[0][l] = CNP_0[1][l] / (2.334 - 0.01334 * CNP_0[1][1])
        result[1][l] = CNP_0[1][l]
        result[2][l] = CNP_0[1][l] / (0.4036 + 0.0059 * CNP_0[1][l])
    return result


@memoize
def CNP_f(NRur, NUrb, CNP_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((3, nlu))
    result[0] = CNP_0[1] / (2.334 - 0.01334 * CNP_0[1][1])
    result[1] = CNP_0[1]
    result[2] = CNP_0[1] / (0.4036 + 0.0059 * CNP_0[1])
    return result



from gwlfe.Memoization import memoize

try:
    from .InitSnowYesterday_inner_compiled import InitSnowYesterday_inner
except ImportError:
    print("Unable to import compiled InitSnowYesterday_inner, using slower version")
    from gwlfe.Input.WaterBudget.InitSnowYesterday_inner import InitSnowYesterday_inner


@memoize
def InitSnowYesterday(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    return InitSnowYesterday_inner(NYrs, DaysMonth, InitSnow_0, Temp, Prec)



from numba.pycc import CC
from numpy import zeros

cc = CC('Percolation_inner_compiled')


@cc.export('Percolation_inner', '(int64, float64, int64[:,::1], float64, float64[:,:,::1], float64[:,:,::1])')
def Percolation_inner(NYrs, UnsatStor_0, DaysMonth, MaxWaterCap, infiltration, et):
    result = zeros((NYrs, 12, 31))
    percolation = zeros((NYrs, 12, 31))
    unsatstor_carryover = UnsatStor_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = unsatstor_carryover
                result[Y][i][j] = result[Y][i][j] + infiltration[Y][i][j]
                if et[Y][i][j] >= result[Y][i][j]:
                    result[Y][i][j] = 0
                else:
                    result[Y][i][j] = result[Y][i][j] - et[Y][i][j]
                if result[Y][i][j] > MaxWaterCap:
                    percolation[Y][i][j] = result[Y][i][j] - MaxWaterCap
                    result[Y][i][j] = MaxWaterCap
                else:
                    pass
                unsatstor_carryover = result[Y][i][j]
    return percolation



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.AdjQTotal import AdjQTotal
from gwlfe.MultiUse_Fxns.Discharge.AdjQTotal import AdjQTotal_f
from gwlfe.MultiUse_Fxns.Discharge.QTotal import QTotal
from gwlfe.MultiUse_Fxns.Discharge.QTotal import QTotal_f


@memoize
def DayRunoff(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
              Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN):
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adj_q_total = AdjQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                            Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    q_total = QTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, CN)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if adj_q_total[Y][i][j] > 0:
                        result[Y][i][j] = adj_q_total[Y][i][j]
                    elif q_total[Y][i][j] > 0:
                        result[Y][i][j] = q_total[Y][i][j]
                    else:
                        result[Y][i][j] = 0
                else:
                    pass
    return result


@memoize
def DayRunoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN):
    result = zeros((NYrs, 12, 31))
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adj_q_total = AdjQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN)
    q_total = QTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                       Imper,
                       ISRR, ISRA, CN)
    result[where((Temp > 0) & (water > 0.01) & (adj_q_total > 0))] = adj_q_total[
        where((Temp > 0) & (water > 0) & (adj_q_total > 0))]
    result[where((Temp > 0) & (water > 0.01) & (q_total > 0))] = q_total[
        where((Temp > 0) & (water > 0) & (q_total > 0))]
    return result



from .VariableUnitTest import VariableUnitTest


class TestDeepSeep(VariableUnitTest):
    pass
    #     input_file = open('unittests/input_4.gms', 'r')
    #     self.z = Parser.GmsReader(input_file).read()
    #
    #
    # def test_DeepSeep(self):
    #     z = self.z
    #     np.testing.assert_array_almost_equal(
    #         DeepSeep.DeepSeep_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
    #                             z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
    #                             z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef),
    #         DeepSeep.DeepSeep(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
    #                             z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
    #                             z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef), decimal=7)



from numpy import zeros

from gwlfe.DailyArrayConverter import get_value_for_yesterday
from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.AMC5 import AMC5, AMC5_yesterday
from gwlfe.Input.WaterBudget.GrowFactor import GrowFactor
from gwlfe.Input.WaterBudget.GrowFactor import GrowFactor_f
from gwlfe.Input.WaterBudget.Melt import Melt
from gwlfe.Input.WaterBudget.Melt_1 import Melt_1_f
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.CNI import CNI, CNI_f

try:
    from .CNumImperv_inner_compiled import CNumImperv_inner
except ImportError:
    print("Unable to import compiled CNumImper_inner, using slower version")
    from gwlfe.MultiUse_Fxns.Runoff.CNumImperv_inner import CNumImperv_inner


@memoize
def CNumImperv(NYrs, NRur, NUrb, DaysMonth, InitSnow_0, Temp, Prec, CNI_0, Grow_0, AntMoist_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))
    cni = CNI(NRur, NUrb, CNI_0)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    melt = Melt(NYrs, DaysMonth, Temp, InitSnow_0, Prec)
    grow_factor = GrowFactor(Grow_0)
    amc5 = AMC5(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0)

    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        for l in range(NRur, nlu):
                            if cni[1][l] > 0:
                                if melt[Y][i][j] <= 0:
                                    if grow_factor[i] > 0:
                                        # Growing season
                                        if get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) >= 5.33:
                                            result[Y][i][j][l] = cni[2][l]
                                        elif get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) < 3.56:
                                            result[Y][i][j][l] = cni[0][l] + (
                                                    cni[1][l] - cni[0][l]) * get_value_for_yesterday(amc5, 0, Y, i, j,
                                                                                                     DaysMonth) / 3.56
                                        else:
                                            result[Y][i][j][l] = cni[1][l] + (cni[2][l] - cni[1][l]) * (
                                                    get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) - 3.56) / 1.77
                                    else:
                                        # Dormant season
                                        if get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) >= 2.79:
                                            result[Y][i][j][l] = cni[2][l]
                                        elif get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) < 1.27:
                                            result[Y][i][j][l] = cni[0][l] + (
                                                    cni[1][l] - cni[0][l]) * get_value_for_yesterday(amc5, 0, Y, i, j,
                                                                                                     DaysMonth) / 1.27
                                        else:
                                            result[Y][i][j][l] = cni[1][l] + (cni[2][l] - cni[1][l]) * (
                                                    get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) - 1.27) / 1.52
                                else:
                                    result[Y][i][j][l] = cni[2][l]
    return result


def CNumImperv_f(NYrs, NRur, NUrb, DaysMonth, InitSnow_0, Temp, Prec, CNI_0, Grow_0, AntMoist_0):
    nlu = NLU(NRur, NUrb)
    cni = CNI_f(NRur, NUrb, CNI_0)
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    melt = Melt_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    grow_factor = GrowFactor_f(Grow_0)
    amc5 = AMC5_yesterday(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0)
    return CNumImperv_inner(NYrs, NRur, DaysMonth, Temp, nlu, cni, water, melt, grow_factor, amc5)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.Stream import UrbLoadRed


class TestUrbLoadRed(VariableUnitTest):

    def test_UrbLoadRed(self):
        z = self.z
        # UrbLoadRed.UrbLoadRed_1(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
        #                         z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
        #                         z.PctAreaInfil, z.Nqual, z.Storm, z.UrbBMPRed)
        # UrbLoadRed.UrbLoadRed_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
        #                         z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
        #                         z.PctAreaInfil, z.Nqual, z.Storm, z.UrbBMPRed)
        # UrbLoadRed.UrbLoadRed_3(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
        #                         z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
        #                         z.PctAreaInfil, z.Nqual, z.Storm, z.UrbBMPRed)
        np.testing.assert_array_almost_equal(
            UrbLoadRed.UrbLoadRed_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                    z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
                                    z.PctAreaInfil, z.Nqual, z.Storm, z.UrbBMPRed),
            UrbLoadRed.UrbLoadRed(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                  z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
                                  z.PctAreaInfil, z.Nqual, z.Storm, z.UrbBMPRed), decimal=7)



from numpy import sum
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal import AdjUrbanQTotal_f
from gwlfe.Output.Loading.DisSurfLoad import DisSurfLoad
from gwlfe.Output.Loading.DisSurfLoad import DisSurfLoad_f
from gwlfe.Output.Loading.SurfaceLoad_1 import SurfaceLoad_1
from gwlfe.Output.Loading.SurfaceLoad_1 import SurfaceLoad_1_f


@memoize
def NetSolidLoad(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                 Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                 LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf):
    result = zeros((NYrs, 12, 31, Nqual))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    nlu = NLU(NRur, NUrb)
    adjurbanqtotal = AdjUrbanQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    dissurfaceload = DisSurfLoad(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Nqual, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                 Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, LoadRateImp, LoadRatePerv,
                                 Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    surfaceload_1 = SurfaceLoad_1(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp, LoadRatePerv,
                                  Storm, UrbBMPRed, FilterWidth, PctStrmBuf)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if adjurbanqtotal[Y][i][j] > 0.001:
                        for l in range(NRur, nlu):
                            for q in range(Nqual):
                                result[Y][i][j][q] += surfaceload_1[Y][i][j][l][q] - dissurfaceload[Y][i][j][l][q]
                    else:
                        pass
                else:
                    pass
    return result


def NetSolidLoad_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                   Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                   LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu - NRur, Nqual))
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    adjurbanqtotal = AdjUrbanQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil)
    dissurfaceload = DisSurfLoad_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Nqual, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                   Grow_0, CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, LoadRateImp,
                                   LoadRatePerv, Storm, UrbBMPRed, DisFract, FilterWidth, PctStrmBuf)
    surfaceload_1 = SurfaceLoad_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0,
                                    CNP_0, Imper, ISRR, ISRA, Qretention, PctAreaInfil, Nqual, LoadRateImp,
                                    LoadRatePerv,
                                    Storm, UrbBMPRed, FilterWidth, PctStrmBuf)
    nonzero = where((Temp > 0) & (water > 0.01) & (adjurbanqtotal > 0.001))
    result[nonzero] = surfaceload_1[nonzero] - dissurfaceload[nonzero]
    return sum(result, axis=3)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import RurQRunoff


class TestRurQRunoff(VariableUnitTest):
    def test_RurQRunoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            RurQRunoff.RurQRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb,
                                    z.CN, z.Grow_0),
            np.swapaxes(
                RurQRunoff.RurQRunoff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb,
                                      z.CN, z.Grow_0), 1, 2)[:, :, :z.NRur], decimal=7)



from numpy import sum

from gwlfe.Memoization import memoize


# RurAreaTotal is faster
@memoize
def RurAreaTotal(NRur, Area):
    result = 0
    for l in range(NRur):
        result += Area[l]
    return result


@memoize
def RurAreaTotal_f(NRur, Area):
    return sum(Area[0:NRur])



from numpy import sum
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal_1 import UrbanQTotal_1
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal_1 import UrbanQTotal_1_f


@memoize
def UrbanRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA):
    result = zeros((NYrs, 12))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urbanqtotal_1 = UrbanQTotal_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper,
                                  ISRR, ISRA)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    result[Y][i] += urbanqtotal_1[Y][i][j]
                else:
                    pass
    return result


def UrbanRunoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                  CNP_0, Imper, ISRR, ISRA):
    return sum(UrbanQTotal_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                               CNP_0, Imper, ISRR, ISRA), axis=2)



from numpy import repeat
from numpy import reshape
from numpy import zeros

from gwlfe.AFOS.GrazingAnimals.Losses.GRLossN import GRLossN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLossN import GRLossN_f
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostBarnN import GRLostBarnN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostBarnN import GRLostBarnN_f
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostManN import GRLostManN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostManN import GRLostManN_f
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import GRStreamN
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import GRStreamN_f
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import NGLostBarnN
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import NGLostBarnN_f
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostManN import NGLostManN
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostManN import NGLostManN_f
from gwlfe.Memoization import memoize


def AnimalN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
            NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN,
            RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate):
    result = zeros((NYrs, 12))
    ng_lost_man_n = NGLostManN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate,
                               Prec, DaysMonth,
                               NGPctSoilIncRate)
    gr_lost_man_n = GRLostManN(NYrs, GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRAppNRate,
                               Prec, DaysMonth, GRPctSoilIncRate)
    ng_lost_barn_n = NGLostBarnN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                                 Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    gr_lost_barn_n = GRLostBarnN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                                 GRBarnNRate, Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    gr_loss_n = GRLossN(NYrs, PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                        GrazingNRate, Prec,
                        DaysMonth)
    gr_stream_n = GRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (ng_lost_man_n[Y][i]
                            + gr_lost_man_n[Y][i]
                            + ng_lost_barn_n[Y][i]
                            + gr_lost_barn_n[Y][i]
                            + gr_loss_n[Y][i]
                            + gr_stream_n[i])
    return result


@memoize
def AnimalN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
              NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN,
              RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate):
    ng_lost_man_n = NGLostManN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate,
                                 Prec, DaysMonth, NGPctSoilIncRate)
    gr_lost_man_n = GRLostManN_f(NYrs, GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRAppNRate,
                                 Prec, DaysMonth, GRPctSoilIncRate)
    ng_lost_barn_n = NGLostBarnN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                   NGBarnNRate, Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    gr_lost_barn_n = GRLostBarnN_f(NYrs, Prec, DaysMonth, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                   GRPctManApp, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, RunContPct,
                                   RunConCoeffN)
    gr_loss_n = GRLossN_f(NYrs, PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                          GrazingNRate, Prec,
                          DaysMonth)
    gr_stream_n = reshape(
        repeat(GRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN),
               repeats=NYrs, axis=0), (NYrs, 12))
    return ng_lost_man_n + gr_lost_man_n + ng_lost_barn_n + gr_lost_barn_n + gr_loss_n + gr_stream_n



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.Stream import FilterEff


class TestFilterEff(VariableUnitTest):

    def test_FilterEff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            FilterEff.FilterEff_f(z.FilterWidth),
            FilterEff.FilterEff(z.FilterWidth), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import ErosWashoff


class TestErosWashoff(VariableUnitTest):

    def test_ErosWashoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            ErosWashoff.ErosWashoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.Acoef, z.KF, z.LS,
                                      z.C, z.P, z.Area),
            np.swapaxes(
                ErosWashoff.ErosWashoff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Acoef,
                                        z.KF,
                                        z.LS, z.C, z.P, z.Area)[:, :z.NRur], 1, 2), decimal=7)



import os
import unittest

from gwlfe import Parser


class VariableUnitTest(unittest.TestCase):
    def setUp(self):
        self.basepath = os.path.abspath(os.path.join(__file__, '../'))
        testgms = os.path.join(self.basepath, 'input_4.gms')
        with open(testgms, 'r') as input_file:
            self.z = Parser.GmsReader(input_file).read()



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import AdjQTotal


class TestAdjQTotal(VariableUnitTest):

    def test_AdjQTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AdjQTotal.AdjQTotal_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                  z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                  z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.n25b, z.CN),
            AdjQTotal.AdjQTotal(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.n25b, z.CN), decimal=7)



# from Timer import time_function
from numpy import zeros

from gwlfe.Input.LandUse.TotAreaMeters import TotAreaMeters
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.StreamFlowLE import StreamFlowLE
from gwlfe.MultiUse_Fxns.Discharge.StreamFlowLE import StreamFlowLE_f


@memoize
def StreamFlowVol(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef
                  , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                  GroundWithdrawal):
    # CALCULATE THE VOLUMETRIC STREAM Flow
    result = zeros((NYrs, 12))
    streamflowle = StreamFlowLE(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0, Imper,
                                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                                SeepCoef
                                , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                                StreamWithdrawal, GroundWithdrawal)
    totareameters = TotAreaMeters(NRur, NUrb, Area)
    for Y in range(NYrs):
        for i in range(12):
            # CALCULATE THE VOLUMETRIC STREAM Flow
            result[Y][i] = ((streamflowle[Y][i] / 100) * totareameters) / (86400 * DaysMonth[Y][i])
    return result


def StreamFlowVol_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                    Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                    GroundWithdrawal):
    streamflowle = StreamFlowLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                  RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                                  PointFlow, StreamWithdrawal, GroundWithdrawal)
    totareameters = TotAreaMeters(NRur, NUrb, Area)
    return streamflowle / 100 * totareameters / (86400 * DaysMonth)



from .VariableUnitTest import VariableUnitTest


class TestAgAreaTotal(VariableUnitTest):

    def test_AgAreaTotal(self):
        pass
        # z = self.z
        # np.testing.assert_array_almost_equal(
        #     AgAreaTotal.AgAreaTotal_f(z.Landuse, z.Area),
        #     AgAreaTotal.AgAreaTotal(z.NRur, z.Landuse, z.Area), decimal=7)



from numpy import zeros

from .InitGrN import InitGrN
from .InitGrN import InitGrN_f
from gwlfe.Memoization import memoize


def GRAppManN(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = zeros((12,))
    init_gr_n = InitGrN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result[i] = GRPctManApp[i] * init_gr_n
    return result


@memoize
def GRAppManN_f(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    init_gr_n = InitGrN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    return GRPctManApp * init_gr_n



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import LuDisLoad


class TestLuDisLoad(VariableUnitTest):

    def test_LuDisLoad(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            LuDisLoad.LuDisLoad_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Nqual, z.NRur, z.NUrb, z.Area,
                                  z.CNI_0, z.AntMoist_0,
                                  z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                                  z.LoadRateImp, z.LoadRatePerv,
                                  z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf),
            LuDisLoad.LuDisLoad(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur,
                                z.NUrb, z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0,
                                z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                                z.Nqual, z.LoadRateImp, z.LoadRatePerv,
                                z.Storm, z.UrbBMPRed, z.DisFract,
                                z.FilterWidth, z.PctStrmBuf)[:, z.NRur:], decimal=7)



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.InitSnow import InitSnow
from gwlfe.Input.WaterBudget.InitSnowYesterday import InitSnowYesterday


def Melt(NYrs, DaysMonth, Temp, InitSnow_0, Prec):
    result = zeros((NYrs, 12, 31))
    init_snow = InitSnow(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    init_snow_yesterday = InitSnow_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and init_snow_yesterday > 0.001:
                    result[Y][i][j] = 0.45 * Temp[Y][i][j]
                else:
                    result[Y][i][j] = 0
                init_snow_yesterday = init_snow[Y][i][j]
    return result


def Melt_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec):
    init_snow_yesterday = InitSnowYesterday(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    return where((Temp > 0) & (init_snow_yesterday > 0.001), 0.45 * Temp, 0)



from gwlfe.Input.Animals.AEU import AEU
from gwlfe.Input.Animals.AEU import AEU_f
from gwlfe.Input.LandUse.Urb.PcntUrbanArea import PcntUrbanArea
from gwlfe.Input.LandUse.Urb.PcntUrbanArea import PcntUrbanArea_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.AvCN import AvCN
from gwlfe.MultiUse_Fxns.Runoff.AvCN import AvCN_f


@memoize
def SedAFactor(NumAnimals, AvgAnimalWt, NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area, SedAFactor_0, AvKF, AvSlope,
               SedAAdjust):
    pcnturbanarea = PcntUrbanArea(NRur, NUrb, Area)
    aeu = AEU(NumAnimals, AvgAnimalWt, NRur, NUrb, Area)
    avcn = AvCN(NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area)
    # Recalculate Sed A Factor using updated AEU value based on animal data
    result = ((0.00467 * pcnturbanarea) +
              (0.000863 * aeu) +
              (0.000001 * avcn) +
              (0.000425 * AvKF) +
              (0.000001 * AvSlope) - 0.000036) * SedAAdjust

    if result < 0.00001:
        result = 0.00001
    return result


def SedAFactor_f(NumAnimals, AvgAnimalWt, NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area, SedAFactor_0, AvKF, AvSlope,
                 SedAAdjust):
    pcnturbanarea = PcntUrbanArea_f(NRur, NUrb, Area)
    aeu = AEU_f(NumAnimals, AvgAnimalWt, Area)
    avcn = AvCN_f(NRur, NUrb, CNI_0, CNP_0, CN, Imper, Area)
    # Recalculate Sed A Factor using updated AEU value based on animal data
    result = ((0.00467 * pcnturbanarea) +
              (0.000863 * aeu) +
              (0.000001 * avcn) +
              (0.000425 * AvKF) +
              (0.000001 * AvSlope) - 0.000036) * SedAAdjust

    if result < 0.00001:
        result = 0.00001
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.LandUse import RurAreaTotal


class TestRurAreaTotal(VariableUnitTest):

    def test_RurAreaTotal(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            RurAreaTotal.RurAreaTotal_f(z.NRur, z.Area),
            RurAreaTotal.RurAreaTotal(z.NRur, z.Area), decimal=7)



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.Runoff import Runoff
from gwlfe.MultiUse_Fxns.Runoff.Runoff import Runoff_f


def AvRunoff(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
             Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity):
    result = zeros(12)
    runoff = Runoff(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                    Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += runoff[Y][i] / NYrs
    return result


@memoize
def AvRunoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
               Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity):
    return sum(Runoff_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                        Imper, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, Landuse, TileDrainDensity),
               axis=0) / NYrs



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import Flow


class TestFlow(VariableUnitTest):

    def test_Flow(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Flow.Flow_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                        z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef),
            Flow.Flow(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                      z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                      z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef), decimal=7)



from numpy import sum

from gwlfe.Memoization import memoize


def TotLAEU(NumAnimals, AvgAnimalWt):
    result = 0
    aeu3 = (NumAnimals[5] * AvgAnimalWt[5]) / 1000
    aeu4 = (NumAnimals[4] * AvgAnimalWt[4]) / 1000
    aeu5 = (NumAnimals[6] * AvgAnimalWt[6]) / 1000
    aeu6 = (NumAnimals[0] * AvgAnimalWt[0]) / 1000
    aeu7 = (NumAnimals[1] * AvgAnimalWt[1]) / 1000
    result += aeu3 + aeu4 + aeu5 + aeu6 + aeu7
    return result


@memoize
def TotLAEU_f(NumAnimals, AvgAnimalWt):
    return sum(NumAnimals[[0, 1, 4, 5, 6]] * AvgAnimalWt[[0, 1, 4, 5, 6]] / 1000)



import numpy as np
from numpy import zeros

from gwlfe.Input.WaterBudget.Percolation import Percolation
from gwlfe.Input.WaterBudget.Percolation import Percolation_f
from gwlfe.Memoization import memoize

try:
    from .DeepSeep_inner_compiled import DeepSeep_inner
except ImportError:
    print("Unable to import compiled DeepSeep_inner, using slower version")
    from gwlfe.Input.WaterBudget.DeepSeep_inner import DeepSeep_inner


def SatStor_accumulate(DaysMonth, NYrs, RecessionCoef, SeepCoef, percolation, SatStor_0=0):
    result = zeros((NYrs, 12, 31))
    grflow = zeros((NYrs, 12, 31))
    deepseep = zeros((NYrs, 12, 31))

    satstor_carryover = SatStor_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = satstor_carryover
                grflow[Y][i][j] = RecessionCoef * result[Y][i][j]
                deepseep[Y][i][j] = SeepCoef * result[Y][i][j]
                result[Y][i][j] = result[Y][i][j] + percolation[Y][i][j] - grflow[Y][i][j] - deepseep[Y][i][j]
                if result[Y][i][j] < 0:
                    result[Y][i][j] = 0
                satstor_carryover = result[Y][i][j]
    return result


@memoize
def SatStor(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
            ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    percolation = Percolation(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper,
                              ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap)
    return SatStor_accumulate(DaysMonth, NYrs, RecessionCoef, SatStor_0, SeepCoef, percolation)


def SatStor_accumulate_fast(DaysMonth, NYrs, RecessionCoef, SeepCoef, percolation, SatStor_0=0, ):
    mult = RecessionCoef + SeepCoef
    percolation = percolation.flatten()
    result = np.zeros_like(percolation)
    result[0] = SatStor_0
    it = np.nditer([percolation, result], flags=['external_loop'],
                   op_flags=[['readonly'], ['readwrite']],
                   )
    it.operands[1][...] = 0
    for per, st in it:
        st[...] = np.clip(st + per - (mult * st), 0, np.inf)
    return it.operands[1]


def SatStor_accumulate_test():
    sz = (300, 12, 31)
    DaysMonth = np.zeros((sz[0], sz[1]), dtype='int')
    DaysMonth[:, :] = 31
    RecessionCoef, SeepCoef = (.3, .9)
    percolation = np.random.normal(size=sz)
    res1 = SatStor_accumulate(DaysMonth, 300, RecessionCoef, SeepCoef, percolation, SatStor_0=0).flatten()


@memoize
def SatStor_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
              ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    percolation = Percolation_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap)
    return DeepSeep_inner(NYrs, SatStor_0, DaysMonth, RecessionCoef, SeepCoef, percolation)[2]



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import LuTotNitr_1


class TestLuTotNitr_1(VariableUnitTest):

    def test_LuTotNitr_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            LuTotNitr_1.LuTotNitr_1_f(z.NYrs, z.NRur, z.NUrb, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0,
                                      z.CN, z.Grow_0, z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth,
                                      z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0,
                                      z.KF,
                                      z.LS, z.C, z.P, z.SedNitr, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
                                      z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed,
                                      z.FilterWidth, z.PctStrmBuf, z.Acoef, z.CNI_0, z.Nqual, z.ShedAreaDrainLake,
                                      z.RetentNLake, z.AttenFlowDist, z.AttenFlowVel, z.AttenLossRateN),
            LuTotNitr_1.LuTotNitr_1(z.NYrs, z.NRur, z.NUrb, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0,
                                    z.CN, z.Grow_0, z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth,
                                    z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF,
                                    z.LS, z.C, z.P, z.SedNitr, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
                                    z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed,
                                    z.FilterWidth, z.PctStrmBuf, z.Acoef, z.CNI_0, z.Nqual, z.ShedAreaDrainLake,
                                    z.RetentNLake, z.AttenFlowDist, z.AttenFlowVel, z.AttenLossRateN)[:, :],
            decimal=7)



from numpy import sum


def TotAEU(NumAnimals, AvgAnimalWt):
    result = 0
    aeu1 = ((NumAnimals[2] / 2) * (AvgAnimalWt[2]) / 1000) + ((NumAnimals[3] / 2) * (AvgAnimalWt[3]) / 1000)
    aeu2 = (NumAnimals[7] * AvgAnimalWt[7]) / 1000
    aeu3 = (NumAnimals[5] * AvgAnimalWt[5]) / 1000
    aeu4 = (NumAnimals[4] * AvgAnimalWt[4]) / 1000
    aeu5 = (NumAnimals[6] * AvgAnimalWt[6]) / 1000
    aeu6 = (NumAnimals[0] * AvgAnimalWt[0]) / 1000
    aeu7 = (NumAnimals[1] * AvgAnimalWt[1]) / 1000
    result += aeu1 + aeu2 + aeu3 + aeu4 + aeu5 + aeu6 + aeu7
    return result


def TotAEU_f(NumAnimals, AvgAnimalWt):
    aeu = NumAnimals * AvgAnimalWt / 1000
    aeu[2:4] /= 2
    return sum(aeu)



from numpy import zeros

from gwlfe.Input.LandUse.Ag.AGSTRM import AGSTRM
from gwlfe.Input.LandUse.Ag.AGSTRM import AGSTRM_f
from gwlfe.Output.Loading.StreamBankN import StreamBankN
from gwlfe.Output.Loading.StreamBankN import StreamBankN_f


def NFEN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
         CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
         UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
         RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
         TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
         NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
         AvSlope, SedAAdjust, StreamLength, AgLength,
         n42, SedNitr, BankNFrac, n45, n69):
    result = zeros((NYrs, 12))
    streambank_n = StreamBankN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                               CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                               UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                               TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                               NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                               AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)

    agstrm = AGSTRM(AgLength, StreamLength)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = 0
            if n42 > 0:
                result[Y][i] = (n45 / n42) * streambank_n[Y][i] * agstrm * n69
    return result


def NFEN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
           CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
           UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
           RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
           TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
           NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
           AvSlope, SedAAdjust, StreamLength, AgLength,
           n42, SedNitr, BankNFrac, n45, n69):
    if n42 > 0:
        agstrm = AGSTRM_f(AgLength, StreamLength)
        streambank_n = StreamBankN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                     CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                     UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                     RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                     TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                                     NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                                     AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)
        return (n45 / n42) * streambank_n * agstrm * n69
    else:
        return zeros((NYrs, 12))



from numpy import maximum
from numpy import zeros

from gwlfe.BMPs.Stream.SEDFEN import SEDFEN
from gwlfe.BMPs.Stream.SEDFEN import SEDFEN_f
from gwlfe.BMPs.Stream.SEDSTAB import SEDSTAB
from gwlfe.BMPs.Stream.SEDSTAB import SEDSTAB_f
from gwlfe.BMPs.Stream.SURBBANK import SURBBANK
from gwlfe.BMPs.Stream.SURBBANK import SURBBANK_f
from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.StreamBankEros import StreamBankEros
from gwlfe.Output.Loading.StreamBankEros import StreamBankEros_f as StreamBankEros_f_actual


@memoize
def StreamBankEros_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                     Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                     GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                     SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab):
    result = zeros((NYrs, 12))
    streambankeros = StreamBankEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0,
                                    CNP_0, Imper,
                                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                    RecessionCoef, SeepCoef
                                    , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                                    StreamWithdrawal, GroundWithdrawal
                                    , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                                    SedAAdjust, StreamLength)
    sedstab = SEDSTAB(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                      CNP_0, Imper,
                      ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                      RecessionCoef, SeepCoef
                      , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                      StreamWithdrawal, GroundWithdrawal
                      , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                      SedAAdjust, StreamLength, n42b, n46c, n85d)
    sedfen = SEDFEN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                    CNP_0, Imper,
                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                    RecessionCoef, SeepCoef
                    , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                    StreamWithdrawal, GroundWithdrawal
                    , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                    SedAAdjust, StreamLength, AgLength, n42, n45, n85)
    surbbank = SURBBANK(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                        Imper,
                        ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef
                        , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                        GroundWithdrawal
                        , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope, SedAAdjust,
                        StreamLength
                        , UrbBankStab, n42b, n85d)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = streambankeros[Y][i] - (sedstab[Y][i] + sedfen[Y][i] + surbbank[Y][i])
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def StreamBankEros_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                       Imper,
                       ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                       Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                       GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                       SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab):
    streambankeros = StreamBankEros_f_actual(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                                             AntMoist_0, Grow_0,
                                             CNP_0, Imper,
                                             ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                             RecessionCoef, SeepCoef
                                             , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                                             StreamWithdrawal, GroundWithdrawal
                                             , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                                             SedAAdjust, StreamLength)

    sedstab = SEDSTAB_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                        CNP_0, Imper,
                        ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                        RecessionCoef, SeepCoef
                        , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                        StreamWithdrawal, GroundWithdrawal
                        , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                        SedAAdjust, StreamLength, n42b, n46c, n85d)
    sedfen = SEDFEN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                      CNP_0, Imper,
                      ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                      RecessionCoef, SeepCoef
                      , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                      StreamWithdrawal, GroundWithdrawal
                      , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                      SedAAdjust, StreamLength, AgLength, n42, n45, n85)
    surbbank = SURBBANK_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                          Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                          SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                          StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                          AvKF, AvSlope, SedAAdjust, StreamLength, UrbBankStab, n42b, n85d)
    return maximum(streambankeros - (sedstab + sedfen + surbbank), 0)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import GRAppManN


class TestGrAppManN(VariableUnitTest):
    def test_GrAppManN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRAppManN.GRAppManN(z.GRPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            GRAppManN.GRAppManN_f(z.GRPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            decimal=7)



from numpy import zeros

from gwlfe.DailyArrayConverter import get_value_for_yesterday
from gwlfe.Input.WaterBudget.AMC5 import AMC5, AMC5_yesterday
from gwlfe.Input.WaterBudget.GrowFactor import GrowFactor
from gwlfe.Input.WaterBudget.GrowFactor import GrowFactor_f
from gwlfe.Input.WaterBudget.Melt import Melt
from gwlfe.Input.WaterBudget.Melt_1 import Melt_1_f
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.NewCN import NewCN, NewCN_f

try:
    from .CNum_inner_compiled import CNum_inner
except ImportError:
    print("Unable to import compiled CNum_inner, using slower version")
    from gwlfe.MultiUse_Fxns.Runoff.CNum_inner import CNum_inner


@memoize
def CNum(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, CN, NRur, NUrb, Grow_0):
    result = zeros((NYrs, 12, 31, 10))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    melt = Melt(NYrs, DaysMonth, Temp, InitSnow_0, Prec)  # I think this should be Melt_1
    grow_factor = GrowFactor(Grow_0)
    amc5 = AMC5(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0)
    new_cn = NewCN(NRur, NUrb, CN)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        if CN[l] > 0:
                            if melt[Y][i][j] <= 0:
                                if grow_factor[i] > 0:
                                    # growing season
                                    if get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) >= 5.33:
                                        result[Y][i][j][l] = new_cn[2][l]
                                    elif get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) < 3.56:
                                        result[Y][i][j][l] = new_cn[0][l] + (
                                                CN[l] - new_cn[0][l]) * get_value_for_yesterday(amc5, 0, Y, i, j,
                                                                                                DaysMonth) / 3.56
                                    else:
                                        result[Y][i][j][l] = CN[l] + (new_cn[2][l] - CN[l]) * (
                                                get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) - 3.56) / 1.77
                                else:
                                    # dormant season
                                    if get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) >= 2.79:
                                        result[Y][i][j][l] = new_cn[2][l]
                                    elif get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) < 1.27:
                                        result[Y][i][j][l] = new_cn[0][l] + (
                                                CN[l] - new_cn[0][l]) * get_value_for_yesterday(amc5, 0, Y, i, j,
                                                                                                DaysMonth) / 1.27
                                    else:
                                        result[Y][i][j][l] = CN[l] + (new_cn[2][l] - CN[l]) * (
                                                get_value_for_yesterday(amc5, 0, Y, i, j, DaysMonth) - 1.27) / 1.52
                            else:
                                result[Y][i][j][l] = new_cn[2][l]
    return result


# CNUM_f is faster than CNUM_1. CNUM_1 is
@memoize
def CNum_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, CN, NRur, NUrb, Grow_0):
    melt_pest = Melt_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    newcn = NewCN_f(NRur, NUrb, CN)
    amc5 = AMC5_yesterday(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0)
    grow_factor = GrowFactor_f(Grow_0)
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    return CNum_inner(NYrs, DaysMonth, Temp, CN, NRur, melt_pest, newcn, amc5, grow_factor, water)
